import tkinter as tk

root = tk.Tk()


def report_event_details(event):
    print(event.state, event.keysym, event.keycode)


text = tk.Text(root)
text.grid()
text.bind("<Key>", report_event_details, True)

button = tk.Button(root, text="Dummy")
button.grid()

root.mainloop()



try:
    # framework is running
    from .startup_choice import *
except ImportError as _excp:
    # class is imported by itself
    if (
        'attempted relative import with no known parent package' in str(_excp)
        or 'No module named \'omfit_classes\'' in str(_excp)
        or "No module named '__main__.startup_choice'" in str(_excp)
    ):
        from startup_choice import *
    else:
        raise

from omfit_classes.omfit_error import OMFITobjectError

__all__ = ['OMFITpdb']

try:
    import pact.pdb as pdb
except Exception as _excp:
    pdb_error = repr(_excp)

    class OMFITpdb(OMFITobjectError):
        def __init__(self, filename, **kw):
            super().__init__(filename, 'OMFITpdb', error=pdb_error, **kw)


else:

    class OMFITpdb(SortedDict, OMFITobject):
        r"""
        OMFIT class used to interface with PACT pdb files: https://wci.llnl.gov/codes/pact/

        :param filename: filename passed to OMFITobject class

        :param \**kw: keyword dictionary passed to OMFITobject class
        """

        def __init__(self, filename, **kw):
            OMFITobject.__init__(self, filename, **kw)
            SortedDict.__init__(self, sorted=True)
            self.dynaLoad = True

        @dynaLoad
        def load(self):
            with pdb.open(self.filename, 'r') as f:
                for var in f.ls():
                    tmp = f.read(var)
                    sub = self
                    if '@' in var:
                        sub = self.setdefault(var.split('@')[1], SortedDict())
                        var = var.split('@')[0]
                    if isinstance(tmp, (list, tuple)):
                        if isinstance(tmp[0], str):
                            sub[var] = ''.join(tmp).decode('ascii', errors='ignore')
                        else:
                            sub[var] = np.array(tmp)
                            if sub[var].size == 1:
                                sub[var] = sub[var][0]
                            elif len(sub[var].shape) > 1:
                                sub[var] = np.reshape(sub[var][:], sub[var].shape[::-1])
                    else:
                        sub[var] = tmp
                    if isinstance(sub[var], int64):
                        sub[var] = int(sub[var])

        @dynaSave
        def save(self):
            with pdb.open(self.filename, 'w') as f:
                for sub in self:
                    for var in self[sub]:
                        tmp = self[sub][var]
                        if isinstance(tmp, str):
                            tmp = str(tmp)
                        elif isinstance(tmp, np.ndarray):
                            if len(tmp.shape) > 1:
                                tmp = np.reshape(tmp[:], tmp.shape[::-1])
                            tmp = tmp.tolist()
                        try:
                            f.write(var + '@' + sub, tmp)
                        except Exception as _excp:
                            printe('Could not write PDB variable ' + var + '@' + sub)
                            raise



try:
    # framework is running
    from .startup_choice import *
except ImportError as _excp:
    # class is imported by itself
    if (
        'attempted relative import with no known parent package' in str(_excp)
        or 'No module named \'omfit_classes\'' in str(_excp)
        or "No module named '__main__.startup_choice'" in str(_excp)
    ):
        from startup_choice import *
    else:
        raise


from omfit_classes.omfit_hdf5 import OMFIThdf5
from omfit_classes.utils_fusion import available_EFITs
import numpy as np

__all__ = ['OMFITefitCDF', 'available_EFITs']


class OMFITefitCDF(OMFIThdf5):
    r"""
    class used to interface CDF files generated by EFIT++

    :param filename: filename passed to OMFITascii class

    :param \**kw: keyword dictionary passed to OMFITascii class
    """

    @property
    def times(self):
        return self['time']['data']

    def plot(self, time=None):
        """
        Function used to plot input constraints and outputs

        This method is called by .plot() when the object is a CDF file

        :param time: time in seconds

        :return: None
        """

        if time is None:
            time = self.times[0]

        fig = pyplot.gcf()
        fig.set_size_inches(11, 8)

        self.ax = pyplot.subplot(221)
        self.plot_trace('boundaries', title='Boundary', yname='[m]', xname='Time [s]', tgtName='rCoords', sigName='rSigmas')

        self.ax = pyplot.subplot(222)
        self.plot_trace('plasmaCurrent', title='Current', yname='[A]', xname='Time [s]', sigName='sigma')

        self.ax = pyplot.subplot(223)
        self.plot_profile('pressures', time, title='Pressure', yname='[Pa]', rName="rCoords")

        self.ax = pyplot.subplot(224)
        self.plot_profile('mse', time, title='MSE', yname='[Radians]', ylim_min=-5, ylim_max=5)

    def plot_trace(
        self,
        grpName,
        yname=None,
        xname='Radius [m]',
        tgtName="target",
        cmpName="computed",
        sigName="sigmas",
        title=None,
        ylim_min=None,
        ylim_max=None,
    ):
        """

        :param grpName: EFIT++ constraint dictionary name e.g. [input][constraints][grpName]

        :param yname: y-axis figure title

        :param xname: x-axis figure title

        :param tgtName: EFIT++ target profile dictionary name e.g. [input][constraints][grpName][tgtName]

        :param cmpName: EFIT++ computed profile dictionary name e.g. [input][constraints][grpName][cmpName]

        :param sigName: EFIT++ sigma profile dictionary name e.g. [input][constraints][grpName][sigName]

        :param title: figure title

        :param ylim_min: y-axis figure minimum value

        :param ylim_max: y-axis figure maximum value

        :return:
        """

        if grpName not in self['input']['constraints']:
            print(grpName + " constraint does not exist")
            self.ax.set_visible(False)
            return

        # target profiles
        target = np.squeeze(self['input']['constraints'][grpName][tgtName]['data'])
        sigmas = np.squeeze(self['input']['constraints'][grpName][sigName]['data'])

        # computed profiles
        if grpName == 'boundaries':
            computed = np.squeeze(self['output']['separatrixGeometry']['rmidplaneOut']['data'])
        else:
            computed = np.squeeze(self['input']['constraints'][grpName][cmpName]['data'])

        self.ax.ticklabel_format(style='sci', scilimits=(0, 0), axis='y')
        self.ax.errorbar(self.times, target, marker='o', markersize=5, yerr=sigmas, label='Target', color='b')
        self.ax.plot(self.times, computed, marker='x', markersize=5, label='Computed', linewidth=2, color='r')
        self.ax.set_ylim(ylim_min, ylim_max)
        self.ax.legend(title=title, prop={'size': 10}, bbox_to_anchor=[0.98, 0.98], fancybox=True)
        self.ax.set_ylabel(yname)
        self.ax.set_xlabel(xname)
        self.ax.grid(True, linestyle='-', color='0.75', linewidth=0.2)

    def plot_profile(
        self,
        grpName,
        time=None,
        yname=None,
        xname='Radius [m]',
        tgtName="target",
        cmpName="computed",
        sigName="sigmas",
        rName="r",
        title=None,
        ylim_min=None,
        ylim_max=None,
    ):
        '''

        :param grpName: EFIT++ constraint dictionary name e.g. [input][constraints][grpName]

        :param time: single time slice in seconds to plot profile data

        :param yname: y-axis figure title

        :param xname: x-axis figure title

        :param tgtName: EFIT++ target profile dictionary name e.g. [input][constraints][grpName][tgtName]

        :param cmpName: EFIT++ computed profile dictionary name e.g. [input][constraints][grpName][cmpName]

        :param sigName: EFIT++ sigma profile dictionary name e.g. [input][constraints][grpName][sigName]

        :param rName: EFIT++ radius profile dictionary name e.g. [input][constraints][grpName][rName]

        :param title: figure title

        :param ylim_min: y-axis figure minimum value

        :param ylim_max: y-axis figure maximum value

        :return:
        '''

        if time is None:
            time = self.times[0]

        if grpName not in self['input']['constraints']:
            print(grpName + " constraint does not exist")
            self.ax.set_visible(False)
            return

        # profile time slice
        tind = np.abs(time - self.times).argmin()

        # target profiles
        target = self['input']['constraints'][grpName][tgtName]['data']
        sigmas = self['input']['constraints'][grpName][sigName]['data']

        # computed profiles
        computed = self['input']['constraints'][grpName][cmpName]['data']

        # radius
        radius = self['input']['constraints'][grpName][rName]['data']

        # separatrix
        rSepOut = self['output']['separatrixGeometry']['rmidplaneOut']['data']

        if len(np.shape(radius)) > 1:
            radius = radius[tind, :]

        self.ax.ticklabel_format(style='sci', scilimits=(0, 0), axis='y')
        self.ax.errorbar(radius, target[tind, :], marker='o', markersize=5, yerr=sigmas[tind, :], label='Target', color='b')
        self.ax.plot(radius, computed[tind, :], marker='x', markersize=5, label='Computed', linewidth=2, color='r')
        self.ax.set_ylim(ylim_min, ylim_max)
        self.ax.plot([rSepOut[tind], rSepOut[tind]], self.ax.get_ylim(), linestyle='--', color='r', label='Separatrix')
        self.ax.legend(title=title + '; t= ' + str(time) + ' s', prop={'size': 10}, bbox_to_anchor=[0.98, 0.98], fancybox=True)
        self.ax.set_ylabel(yname)
        self.ax.set_xlabel(xname)
        self.ax.grid(True, linestyle='-', color='0.75', linewidth=0.2)


############################################
if '__main__' == __name__:
    test_classes_main_header()

    tmp = OMFITefitCDF(OMFITsrc + '/../samples/efitOut.nc')
    print(tmp)



try:
    # framework is running
    from .startup_choice import *
except ImportError as _excp:
    # class is imported by itself
    if (
        'attempted relative import with no known parent package' in str(_excp)
        or 'No module named \'omfit_classes\'' in str(_excp)
        or "No module named '__main__.startup_choice'" in str(_excp)
    ):
        from startup_choice import *
    else:
        raise


from omfit_classes.omfit_ascii import OMFITascii

import numpy as np

__all__ = ['OMFITionorbConfig', 'OMFITionorbBirth', 'OMFITionorbHits', 'OMFITionorbFull']


class OMFITionorbConfig(SortedDict, OMFITascii):
    """
    OMFIT class used to read/write IONORB configuration file

    :param filename: filename

    :param kw: additional parameters passed to the OMFITascii class
    """

    def __init__(self, filename, **kw):
        SortedDict.__init__(self)
        OMFITascii.__init__(self, filename, **kw)
        self.dynaLoad = True

    @dynaLoad
    def load(self):
        with open(self.filename, 'r') as f:
            lines = f.read().split('\n')
        for line in lines:
            line = line.strip()
            if line.startswith('#') or not len(line):
                continue
            else:
                tmp = line.split(' ')
                self[tmp[0]] = ' '.join(tmp[1:])
                try:
                    self[tmp[0]] = eval(self[tmp[0]])
                except Exception:
                    pass

    @dynaSave
    def save(self):
        txt = ''
        for item in self:
            if isinstance(self[item], (tuple, list, np.ndarray)):
                txt += item + ' ' + ', '.join(map(str, self[item])) + '\n'
            else:
                txt += item + ' ' + str(self[item]) + '\n'
        with open(self.filename, 'w') as f:
            f.write(txt)


class OMFITionorbBirth(SortedDict, OMFITascii):
    """
    OMFIT class used to read/write IONORB birth list or profiles

    :param filename: filename

    :param kw: additional parameters passed to the OMFITascii class
    """

    def __init__(self, filename, **kw):
        SortedDict.__init__(self)
        OMFITascii.__init__(self, filename, **kw)
        self.dynaLoad = True

    columns = ['R[m]', 'Phi[Rad]', 'z[m]', 'v_R[m/s]', 'v_phi[m/s]', 'v_z[m/s]']

    @dynaLoad
    def load(self):
        conversion = {'m': 'mass', 'q': 'charge', 'notshine': 'fracpinjdepo', 'nels': 'npart'}
        with open(self.filename, 'Ur') as f:
            lines = f.read().strip().split('\n')
        self['header'] = []
        for i, l in enumerate(lines):
            if l[0] == '#':
                self['header'].append(l)
                continue
            if l.startswith('<start-of-data>'):
                break
            k, v = l.split()
            k = conversion.get(k, k)
            self[k] = v
        data = np.fromstring('\n'.join(lines[i + 1 :]), sep=' ').reshape(-1, len(self.columns))

        for k, name in enumerate(self.columns):
            self[name.split('[')[0]] = data[:, k]

    @dynaSave
    def save2(self):
        txt = '# ionorb birth profile source\n'
        txt += '# version: v2\n'
        txt += 'm %5.5f\n' % self['mass']
        txt += 'q %5.5f\n' % self['charge']
        txt += 'notshine %d\n' % self['fracpinjdepo']
        txt += 'nels %5.3f\n' % self['npart']
        txt += ' ' + ' '.join([x.ljust(13) for x in self.columns]) + '\n'
        txt += '<start-of-data>\n'
        data = np.array([self[name.split('[')[0]] for name in self.columns]).T
        data_string = []
        for row in range(data.shape[0]):
            data_string.append(' ' + ' '.join(['% 8.6e' % x for x in data[row, :]]))
        txt += '\n'.join(data_string)
        with open(self.filename, 'w') as f:
            f.write(txt)


class OMFITionorbHits(SortedDict, OMFITascii):
    """
    OMFIT class used to read the IONORB hits file

    :param filename: filename

    :param kw: additional parameters passed to the OMFITascii class
    """

    def __init__(self, filename, **kw):
        SortedDict.__init__(self)
        OMFITascii.__init__(self, filename, **kw)
        self.dynaLoad = True

    columns = ['ID', 'Time[s]', 'Step', 'Wall_Idx', 'R[m]', 'Phi[Rad]', 'z[m]', 'v_R[m/s]', 'v_phi[m/s]', 'v_z[m/s]']

    @dynaLoad
    def load(self):
        with open(self.filename, 'r') as f:
            lines = f.read().strip().split('\n')
        data = np.array([list(map(float, line.split())) for line in lines[1:]])
        for k, name in enumerate(self.columns):
            self[name.split('[')[0]] = data[:, k]

    def plot(self, **kw):
        kw.setdefault('ls', '')
        kw.setdefault('marker', 'x')
        pyplot.plot(self['R'], self['z'], **kw)


class OMFITionorbFull(SortedDict, OMFITascii):
    """
    OMFIT class used to read the Full Orbit file output by the ionorbiter code

    :param filename: filename

    :param kw: additional parameters passed to the OMFITascii class
    """

    def __init__(self, filename, **kw):
        SortedDict.__init__(self)
        OMFITascii.__init__(self, filename, **kw)
        self.dynaLoad = True

    columns = ['Time[s]', 'Step', 'R[m]', 'Phi[Rad]', 'z[m]']

    @dynaLoad
    def load(self):
        with open(self.filename, 'r') as f:
            lines = f.read().strip().split('\n')
        data = np.array([list(map(float, line.split())) for line in lines[1:]])
        for k, name in enumerate(self.columns):
            self[name.split('[')[0]] = data[:, k]



try:
    # framework is running
    from .startup_choice import *
except ImportError as _excp:
    # class is imported by itself
    if (
        'attempted relative import with no known parent package' in str(_excp)
        or 'No module named \'omfit_classes\'' in str(_excp)
        or "No module named '__main__.startup_choice'" in str(_excp)
    ):
        from startup_choice import *
    else:
        raise

from omfit_classes.omfit_ascii import OMFITascii

import numpy as np

__all__ = ['OMFITgnuplot']


class OMFITgnuplot(OMFITascii, SortedDict):
    r"""
    Class used to parse ascii gnuplot data files
    Gnuplot data indices are split by double empty lines
    Gnuplot data blocks are split by a single empty line

    :param filename: filename passed to OMFITobject class

    :param comment: character

    :param \**kw: keyword dictionary passed to OMFITobject class
    """

    def __init__(self, filename, comment='#', **kw):
        kw['comment'] = comment
        self.comment = comment
        OMFITascii.__init__(self, filename, **kw)
        SortedDict.__init__(self)
        self.dynaLoad = True

    @dynaLoad
    def load(self):
        with open(self.filename, 'r') as f:
            lines = f.read()
        lines = lines.split('\n')
        for k, line in enumerate(lines):
            if self.comment in line:
                lines[k] = line.split(self.comment)[0] + '\n'
        lines = '\n'.join(lines).strip('\n')
        INDEX = lines.split('\n\n\n')
        for kindex, index in enumerate(INDEX):
            BLOCK = index.split('\n\n')
            for kblock, block in enumerate(BLOCK):
                self[kindex, kblock] = np.atleast_2d(np.genfromtxt(StringIO(block)))

    def plot(self, ix=0, iy=1, **kw):
        r"""
        Plot individual data-sets

        :param ix: index of the data column to be used for the X axis

        :param iy: index of the data column to be used for the Y axis

        :param \**kw: parameters passed to the matplotlib plot function
        """
        for item in self:
            if self[item].shape[1] == 1:
                pyplot.plot(self[item][:, ix], **kw)
            elif self[item].shape[1] >= 2:
                pyplot.plot(self[item][:, ix], self[item][:, iy], **kw)


############################################
if __name__ == '__main__':
    test_classes_main_header()

    lines = '''
    1 2 3 4
    5 6 7 8
    %bla
    9 10 11 12
    13 14 15 16


    17 18 19 20 21
    22 23 24 25 26
    '''

    gplot = OMFITgnuplot('gnuplot.txt', fromString=lines, comment='%')
    print(gplot[(1, 0)])
